"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var mockFactory_1 = require("../test/mockFactory");
var utils_1 = require("./utils");
describe("utils", function () {
    var sls;
    beforeEach(function () {
        var slsConfig = {
            service: "my test service",
            provider: "azure",
            functions: mockFactory_1.MockFactory.createTestSlsFunctionConfig(),
        };
        sls = mockFactory_1.MockFactory.createTestServerless();
        Object.assign(sls.service, slsConfig);
    });
    it("resolves handler when handler code is outside function folders", function () {
        sls.service["functions"].hello.handler = "src/handlers/hello.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("../src/handlers/hello.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
    it("resolves handler when code is in function folder", function () {
        sls.service["functions"].hello.handler = "hello/index.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("index.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
    it("resolves handler when code is at the project root", function () {
        sls.service["functions"].hello.handler = "hello.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("../hello.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
    it("should create string from substrings", function () {
        expect(utils_1.Utils.appendSubstrings(2, "abcde", "fghij", "klmno", "pqrst", "uvwxyz", "ab")).toEqual("abfgklpquvab");
    });
    it("should get a timestamp from a name", function () {
        expect(utils_1.Utils.getTimestampFromName("myDeployment-t12345")).toEqual("12345");
        expect(utils_1.Utils.getTimestampFromName("myDeployment-t678987645")).toEqual("678987645");
        expect(utils_1.Utils.getTimestampFromName("-t12345")).toEqual("12345");
        expect(utils_1.Utils.getTimestampFromName("myDeployment-t")).toEqual(null);
        expect(utils_1.Utils.getTimestampFromName("")).toEqual(null);
    });
    it("should get incoming binding", function () {
        expect(utils_1.Utils.getIncomingBindingConfig(mockFactory_1.MockFactory.createTestAzureFunctionConfig())).toEqual({
            http: true,
            "x-azure-settings": mockFactory_1.MockFactory.createTestHttpBinding("in"),
        });
    });
    it("should get outgoing binding", function () {
        expect(utils_1.Utils.getOutgoingBinding(mockFactory_1.MockFactory.createTestAzureFunctionConfig())).toEqual({
            http: true,
            "x-azure-settings": mockFactory_1.MockFactory.createTestHttpBinding("out"),
        });
    });
    describe("runWithRetry", function () {
        it("returns values after 1st run", function () { return __awaiter(_this, void 0, void 0, function () {
            var expected, lastRetry, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        expected = "success";
                        lastRetry = 0;
                        return [4 /*yield*/, utils_1.Utils.runWithRetry(function (retry) {
                                lastRetry = retry;
                                return Promise.resolve(expected);
                            })];
                    case 1:
                        result = _a.sent();
                        expect(lastRetry).toEqual(1);
                        expect(result).toEqual(expected);
                        return [2 /*return*/];
                }
            });
        }); });
        it("returns values after successfully retry (reject promise)", function () { return __awaiter(_this, void 0, void 0, function () {
            var expected, lastRetry, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        expected = "success";
                        lastRetry = 0;
                        return [4 /*yield*/, utils_1.Utils.runWithRetry(function (retry) {
                                lastRetry = retry;
                                if (retry === 1) {
                                    return Promise.reject("rejected");
                                }
                                return Promise.resolve(expected);
                            })];
                    case 1:
                        result = _a.sent();
                        expect(lastRetry).toEqual(2);
                        expect(result).toEqual(expected);
                        return [2 /*return*/];
                }
            });
        }); });
        it("returns values after successfully retry (throw error)", function () { return __awaiter(_this, void 0, void 0, function () {
            var expected, lastRetry, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        expected = "success";
                        lastRetry = 0;
                        return [4 /*yield*/, utils_1.Utils.runWithRetry(function (retry) {
                                lastRetry = retry;
                                if (retry === 1) {
                                    throw new Error("Ooops!");
                                }
                                return Promise.resolve(expected);
                            })];
                    case 1:
                        result = _a.sent();
                        expect(lastRetry).toEqual(2);
                        expect(result).toEqual(expected);
                        return [2 /*return*/];
                }
            });
        }); });
        it("throws error after reties", function () { return __awaiter(_this, void 0, void 0, function () {
            var maxRetries, lastRetry, test;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        maxRetries = 5;
                        lastRetry = 0;
                        test = function () { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, utils_1.Utils.runWithRetry(function (retry) {
                                            lastRetry = retry;
                                            return Promise.reject("rejected");
                                        }, maxRetries, 100)];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        }); };
                        return [4 /*yield*/, expect(test()).rejects.toEqual("rejected")];
                    case 1:
                        _a.sent();
                        expect(lastRetry).toEqual(maxRetries);
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe("wait", function () {
        var setTimeoutMock = jest.fn(function (resolve) { return resolve(); });
        beforeEach(function () {
            global.setTimeout = setTimeoutMock;
        });
        it("waits 1000 by default", function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, utils_1.Utils.wait()];
                    case 1:
                        _a.sent();
                        expect(setTimeoutMock).toBeCalledWith(expect.any(Function), 1000);
                        return [2 /*return*/];
                }
            });
        }); });
        it("waits specified time", function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, utils_1.Utils.wait(2000)];
                    case 1:
                        _a.sent();
                        expect(setTimeoutMock).toBeCalledWith(expect.any(Function), 2000);
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
